local vector = require "vector"
local common = require "common"
local HC = require "HC"
local love = love
local math = math
local setmetatable = setmetatable
local string = string
local print = print

local Ball = {}

if setfenv then
   setfenv(1, Ball) -- for 5.1
else
   _ENV = Ball -- for 5.2
end

local tileset_width = 800
local tileset_height = 600
local ball_tile_width = 16
local ball_tile_height = 16
local ball_radius = ball_tile_width / 2
local x_pos = 0
local y_pos = 96

local platform_starting_x_pos = 150
local platform_starting_y_pos = 500
local platform_height = 16
local ball_x_shift = 28

function Ball:new( o )
   o = o or {}
   setmetatable(o, self)
   self.__index = self
   o.name = "ball"
   o.image = o.image or nil
   o.quad = love.graphics.newQuad( x_pos, y_pos,
				   ball_tile_width, ball_tile_height,
				   tileset_width, tileset_height )
   o.radius = ball_radius
   if o.stuck_to_platform == nil then
      o.stuck_to_platform = true
   end
   o.platform_launch_speed_magnitude =
      o.platform_launch_speed_magnitude or math.sqrt( 350 * 350 + 350 * 350 )
   o.speed = o.speed or vector(0, 0)
   o.separation_from_platform_center = o.separation_from_platform_center or
      vector( -ball_x_shift,
	      (-1) * platform_height / 2 - o.radius - 1 )
   o.position = o.position or
      vector( platform_starting_x_pos + o.separation_from_platform_center.x,
	      platform_starting_y_pos + o.separation_from_platform_center.y )
   o.collisions.shape = o.collisions.collider:circle( o.position.x,
						      o.position.y,
						      o.radius )
   o.collisions.shape.game_object = o
   o.collisions.counter = 0
   o.collisions.increase_speed_after_collision = 1.03
   return o
end

function Ball:update( dt, platform )
   self.position = self.position + self.speed * dt
   if love.keyboard.isDown("up") then
      self.speed = self.speed * 1.05
   elseif love.keyboard.isDown("down") then      
      self.speed = self.speed * 0.9
   end           
   if self.stuck_to_platform then
      self:follow_platform( platform )
   end
   self.collisions.shape:moveTo( self.position:unpack() )
end

function Ball:draw()
   love.graphics.draw( self.image,
   		       self.quad, 
   		       self.position.x - self.radius,
   		       self.position.y - self.radius )
end

function Ball:follow_platform( platform )   
   local platform_center = vector( platform.position.x + platform.width / 2,
				   platform.position.y + platform.height / 2 )
   self.position = platform_center + self.separation_from_platform_center
end

function Ball:launch_from_platform( platform )
   self.stuck_to_platform = false
   local max_angle_from_normal = 30
   local ball_center = vector( self.collisions.shape:center() ) 
   local platform_center = vector( platform.collisions.shape:center() )
   local separation = ( ball_center - platform_center )
   local launch_angle = separation.x / ( platform.width / 2 ) * max_angle_from_normal
   --print( "la", launch_angle )
   local launch_angle_rad = launch_angle / 180 * math.pi      
   local direction = vector( math.sin( launch_angle_rad ),
			     -math.cos( launch_angle_rad ) )
   self.speed = self.platform_launch_speed_magnitude * direction
end

-- Collisions

function Ball:increase_collision_counter()
   self.collisions.counter = self.collisions.counter + 1 
   --print( "Ball collision counter = ", self.collisions.counter )
end

function Ball:increase_speed_after_collision()
   if self.collisions.counter % 5 == 0 then
      self.speed = self.speed * self.collisions.increase_speed_after_collision
   end
end

function Ball:react_on_wall_collision( other, separating_vector )
   self.position = self.position + separating_vector
   self.collisions.shape:moveTo( self.position:unpack() )
   self:normal_rebound( separating_vector )
   self:min_angle_rebound()
   self:increase_speed_after_collision()
   self:increase_collision_counter()   
end

function Ball:react_on_brick_collision( other, separating_vector )
   self.position = self.position + separating_vector
   self.collisions.shape:moveTo( self.position:unpack() )
   self:normal_rebound( separating_vector )
   self:increase_speed_after_collision()
   self:increase_collision_counter()
end

function Ball:react_on_platform_collision( other, separating_vector )
   self.position = self.position + separating_vector
   self.collisions.shape:moveTo( self.position:unpack() )
   self:increase_speed_after_collision()
   if not other.game_object.glued then
      self:platform_rebound( other, separating_vector )
   else
      self.stuck_to_platform = true
      self.platform_launch_speed_magnitude = self.speed:len()
      self.speed = vector( 0, 0 )
      local platform_center = vector( other:center() )
      local ball_center = vector( self.collisions.shape:center() )
      self.separation_from_platform_center = ball_center - platform_center
   end
   self:increase_collision_counter()
end

function Ball:react_on_slow_down_bonus()
   local slowdown = 0.7
   self.speed = self.speed * slowdown
end

function Ball:react_on_accelerate_bonus()
   local accelerate = 1.3
   self.speed = self.speed * accelerate
end

function Ball:normal_rebound( separating_vector )
   local vx, vy = self.speed:unpack()
   local dx, dy = separating_vector.x, separating_vector.y
   local new_vx, new_vy
   if common.big_enough_overlap( dx ) then
      new_vx = -vx
   else
      new_vx = vx
   end
   if common.big_enough_overlap( dy ) then
      new_vy = -vy
   else
      new_vy = vy
   end
   self.speed = vector( new_vx, new_vy )
end

function Ball:min_angle_rebound()
   local min_horizontal_rebound_angle = math.rad( 20 )
   local vx, vy = self.speed:unpack()
   local new_vx, new_vy = vx, vy
   rebound_angle = math.abs( math.atan( vy / vx ) )
   if rebound_angle < min_horizontal_rebound_angle then
      --print( "Small angle rebound" )
      new_vx = common.sign( vx ) * 
	 math.sqrt( vx * vx + vy * vy ) *
	 math.cos( min_horizontal_rebound_angle )
      new_vy = common.sign( vy ) * 
	 math.sqrt( vx * vx + vy * vy ) *
	 math.sin( min_horizontal_rebound_angle )
   end
   self.speed = vector( new_vx, new_vy )
end

function Ball:platform_rebound( other, separating_vector )
   local platform_width = other.game_object.width
   local dx, dy = separating_vector.x, separating_vector.y
   if ( common.big_enough_overlap( dx ) ) then
      self:inverse_speed_horizontal()
      -- todo: when platform moves and hits ball with side
   end
   if ( common.big_enough_overlap( dy ) ) then
      self:inverse_speed_vertical()
      local max_rotation_degrees = 30
      local ball_center = vector( self.collisions.shape:center() ) 
      local platform_center = vector( other:center() )
      local separation = ( ball_center - platform_center )
      local rotation_deg = separation.x / ( platform_width / 2 ) * max_rotation_degrees
      local rotation_rad = rotation_deg / 180 * math.pi      
      self.speed:rotate_inplace( rotation_rad )
   end
end

function Ball:inverse_speed_horizontal()
   self.speed = vector( -1 * self.speed.x, self.speed.y )
end

function Ball:inverse_speed_vertical()
   self.speed = vector( self.speed.x, -1 * self.speed.y )
end


return Ball
