local vector = require "vector"
local common = require "common"
local HC = require "HC"
local love = love
local math = math
local setmetatable = setmetatable
local string = string

local Platform = {}

if setfenv then
   setfenv(1, Platform)
else
   _ENV = Platform
end


local tileset_width = 800
local tileset_height = 600
local platform_small_tile_width = 75
local platform_small_tile_height = 16
local platform_small_tile_x_pos = 0
local platform_small_tile_y_pos = 0
local platform_norm_tile_width = 108
local platform_norm_tile_height = 16
local platform_norm_tile_x_pos = 0
local platform_norm_tile_y_pos = 32
local platform_large_tile_width = 141
local platform_large_tile_height = 16
local platform_large_tile_x_pos = 0
local platform_large_tile_y_pos = 64
local platfrom_glued_x_pos_shift = 192

local platform_starting_x_pos = 150
local platform_starting_y_pos = 500

function Platform:new( o )
   o = o or {}
   setmetatable(o, self)
   self.__index = self
   o.name = "platform"
   o.size = "norm"
   o.width = platform_norm_tile_width
   o.height = platform_norm_tile_height
   o.image = o.image or nil
   o.quad = love.graphics.newQuad( platform_norm_tile_x_pos,
				   platform_norm_tile_y_pos,
				   platform_norm_tile_width,
				   platform_norm_tile_height,
				   tileset_width, tileset_height )
   o.position = vector( platform_starting_x_pos,
			platform_starting_y_pos )
   o.speed = vector(500, 0)
   o.collisions.radius = o.width/2 * 3
   o.collisions.shape = o.collisions.collider:rectangle( o.position.x,
							 o.position.y,
							 o.width,
							 o.height )
   o.collisions.shape.game_object = o
   return o
end

function Platform:update(dt)
   self:follow_mouse( dt )
   self.collisions.shape:moveTo( self.position.x + self.width / 2,
   				 self.position.y + self.height / 2 )
end

function Platform:draw()
   love.graphics.draw( self.image,
		       self.quad,
		       self.position.x,
		       self.position.y )
end

function Platform:follow_mouse( dt )
   local x, y = love.mouse.getPosition()
   local left_wall_plus_half_platform = 34 + self.width / 2
   local right_border_minus_half_platform = 576 - self.width / 2
   if ( x > left_wall_plus_half_platform and
	x < right_border_minus_half_platform ) then
      self.position.x = x - self.width / 2
   elseif x < left_wall_plus_half_platform then
      self.position.x = left_wall_plus_half_platform - self.width / 2
   elseif x > right_border_minus_half_platform then
      self.position.x = right_border_minus_half_platform - self.width / 2
   end
end

function Platform:react_on_wall_collision( other, separating_vector )
   self.position = self.position + separating_vector
   self.collisions.shape:moveTo( self.position.x + self.width / 2,
   				 self.position.y + self.height / 2 )   
end

function Platform:react_on_decrease_bonus()
   self.glued = false
   if self.size == "small" then
      --remove glued tile if it was present
      self.quad = love.graphics.newQuad( platform_small_tile_x_pos,
					 platform_small_tile_y_pos,
					 platform_small_tile_width,
					 platform_small_tile_height,
					 tileset_width, tileset_height )
   elseif self.size == "norm" then
      self.width = platform_small_tile_width
      self.height = platform_small_tile_height
      self.quad = love.graphics.newQuad( platform_small_tile_x_pos,
					 platform_small_tile_y_pos,
					 platform_small_tile_width,
					 platform_small_tile_height,
					 tileset_width, tileset_height )
      self.size = "small"
   elseif self.size == "large" then
      self.width = platform_norm_tile_width
      self.height = platform_norm_tile_height
      self.quad = love.graphics.newQuad( platform_norm_tile_x_pos,
					 platform_norm_tile_y_pos,
					 platform_norm_tile_width,
					 platform_norm_tile_height,
					 tileset_width, tileset_height )
      self.size = "norm"	 
   end
   self.collisions.collider:remove( self.collisions.shape )
   self.collisions.shape = nil
   self.collisions.shape = self.collisions.collider:rectangle(
      self.position.x,
      self.position.y,
      self.width,
      self.height )
   self.collisions.shape.game_object = self
end

function Platform:react_on_increase_bonus()
   self.glued = false 
   if self.size == "small" then
      self.width = platform_norm_tile_width
      self.height = platform_norm_tile_height
      self.quad = love.graphics.newQuad( platform_norm_tile_x_pos,
					 platform_norm_tile_y_pos,
					 platform_norm_tile_width,
					 platform_norm_tile_height,
					 tileset_width, tileset_height )
      self.size = "norm"
   elseif self.size == "norm" then
      self.width = platform_large_tile_width
      self.height = platform_large_tile_height
      self.quad = love.graphics.newQuad( platform_large_tile_x_pos,
					 platform_large_tile_y_pos,
					 platform_large_tile_width,
					 platform_large_tile_height,
					 tileset_width, tileset_height )
      self.size = "large"
   elseif self.size == "large" then
      --remove glued tile it was present
      self.quad = love.graphics.newQuad( platform_large_tile_x_pos,
					 platform_large_tile_y_pos,
					 platform_large_tile_width,
					 platform_large_tile_height,
					 tileset_width, tileset_height )
   end
   self.collisions.collider:remove( self.collisions.shape )
   self.collisions.shape = nil
   self.collisions.shape = self.collisions.collider:rectangle(
      self.position.x,
      self.position.y,
      self.width,
      self.height )
   self.collisions.shape.game_object = self
end

function Platform:react_on_glue_bonus()
   self.glued = true
   if self.size == "small" then
      self.quad = love.graphics.newQuad(
	 platform_small_tile_x_pos + platfrom_glued_x_pos_shift,
	 platform_small_tile_y_pos,
	 platform_small_tile_width,
	 platform_small_tile_height,
	 tileset_width, tileset_height )
   elseif self.size == "norm" then
      self.quad = love.graphics.newQuad(
	 platform_norm_tile_x_pos + platfrom_glued_x_pos_shift,
	 platform_norm_tile_y_pos,
	 platform_norm_tile_width,
	 platform_norm_tile_height,
	 tileset_width, tileset_height )
   elseif self.size == "large" then
      self.quad = love.graphics.newQuad(
	 platform_large_tile_x_pos + platfrom_glued_x_pos_shift,
	 platform_large_tile_y_pos,
	 platform_large_tile_width,
	 platform_large_tile_height,
	 tileset_width, tileset_height )
   end   
end

return Platform
