local Gamestate = require "hump/gamestate"
local gamepaused = gamepaused
local gamefinished = gamefinished
local Platform = require "Platform"
local Ball = require "Ball"
local BallsContainer = require "BallsContainer"
local Brick = require "Brick"
local BricksMap = require "BricksMap"
local Wall = require "Wall"
local WallsContainer = require "WallsContainer"
local Bonus = require "Bonus"
local BonusesContainer = require "BonusesContainer"
local SidePanel = require "SidePanel"
local LifeDisplay = require "LifeDisplay"
local ScoreDisplay = require "ScoreDisplay"
local vector = require "vector"
local HC = require "HC"
local love = love
local math = math
local require = require
local print = print
local pairs = pairs

local game = game or {}
if setfenv then
  setfenv(1, game)
else
   _ENV = game
end

game.state_name = "game"

function game:init()
   
end

function game:enter( previous_state, ... )
   args = ...

   level_sequence = args.level_sequence
   level_number = args.level_number
   if args.level_map then
      level_map = args.level_map
   else
      level_filename =
	 "levels/" .. level_sequence.sequence[ level_number ]
      level_map = require( level_filename )
      level_map.tileset_image_file = level_sequence.tileset_image_file
      level_map.tileset_description = level_sequence.tileset_description
      level_map.tileset_id_to_quad = level_sequence.tileset_id_to_quad
      level_map.human_readable_type_to_quad = level_sequence.human_readable_type_to_quad
   end
   --print( "hello" .. level_map.name )

   collider = args.collider or HC.new( 70 )
   
   platform = args.platform or
      Platform:new{
	 image = love.graphics.newImage( "img/800x600/platform_ball_and_walls.png"),
	 collisions = { collider = collider } }
   balls = args.balls or BallsContainer:new()
   if( #balls.balls == 0 ) then
      balls:add_to_list(
	 Ball:new{
	    image = love.graphics.newImage("img/800x600/platform_ball_and_walls.png"),
	    collisions = { collider = collider } } )
   end
   walls = args.walls or
      WallsContainer:new{
	 image = love.graphics.newImage("img/800x600/platform_ball_and_walls.png"),
	 collisions = { collider = collider } }
   bonuses = args.bonuses or
      BonusesContainer:new{}
   bricksmap = args.bricksmap or 
      BricksMap:new{
	 level_map = level_map,
	 bonus_list = bonuses,
	 collisions = { collider = collider } }
   side_panel = args.side_panel or
      SidePanel:new{}
   life_display = args.life_display or
      LifeDisplay:new{}
   score_display = args.score_display or
      ScoreDisplay:new{}
end

function game:update( dt )
   local updates_per_second = 65
   local accum = 0
   local step = 1.0 / updates_per_second
   accum = accum + dt
   while accum >= step do
      platform:update( step )
      balls:update( step, platform )
      bricksmap:update( step )
      walls:update( step )
      bonuses:update( step )
      life_display:update( step )
      score_display:update( step )
      game:resolve_collisions( step )
      game:switch_to_next_level()
      accum = accum - step
   end
end

function game:draw()
   platform:draw()   
   balls:draw()
   bricksmap:draw()
   walls:draw()
   bonuses:draw()
   side_panel:draw()
   life_display:draw()
   score_display:draw()
end

function game:leave()
   level_sequence, level_number = nil
   level_map, platform, balls, walls = nil
   bonuses, bricksmap = nil
   collider = nil
   --collider:destroy()
end	

function game:switch_to_next_level()
   local no_more_bricks = bricksmap:no_more_bricks()
   if no_more_bricks then
      local next_level = level_number + 1 
      if next_level <= #level_sequence.sequence then
	 Gamestate.switch( game, { level_sequence = level_sequence,
				   level_number = next_level,
				   life_display = life_display,
				   score_display = score_display } )
      else
	 Gamestate.switch( gamefinished )
      end	
   end	
end	

function game:keyreleased( key, code )
   balls:keyreleased( key, code, platform )
   if key == 'escape' then
      Gamestate.switch(gamepaused,
		       { level_sequence = level_sequence,
			 level_number = level_number,
			 collider = collider, 
			 level_map = level_map,
			 platform = platform,
			 balls = balls,
			 walls = walls,
			 bonuses = bonuses,
			 bricksmap = bricksmap,			 
			 life_display = life_display,
			 score_display = score_display,
			 side_panel = side_panel } )
   elseif key == '1' then
      Gamestate.switch( game, { level_sequence = level_sequence,
				level_number = 1 } )
   elseif key == '2' then
      Gamestate.switch( game, { level_sequence = level_sequence,
				level_number = 2 } )
   elseif key == '3' then
      Gamestate.switch( game, { level_sequence = level_sequence,
				level_number = 3 } )
   elseif key == '4' then
      Gamestate.switch( game, { level_sequence = level_sequence,
				level_number = 4 } )
   elseif key == '5' then
      Gamestate.switch( game, { level_sequence = level_sequence,
				level_number = 5 } )
   elseif key == 'w' then
      Gamestate.switch( gamefinished )
   end
end

function game:mousereleased( x, y, button, istouch )
   --print( x, y, button, istouch )
   balls:mousereleased( x, y, button, istouch, platform )
end

function game:resolve_collisions( dt )
   -- Platform
   local collisions = collider:collisions( platform.collisions.shape )
   for other, separating_vector in pairs( collisions ) do
      if other.game_object.name == "wall" then
	 platform:react_on_wall_collision( other, separating_vector )
	 if other.game_object:next_level_portal_active() then
	    game:react_on_next_level_bonus()
	 end
      elseif other.game_object.name == "bonus" then	 
	 if other.game_object.bonustype == 'bonus_blue' then
	    balls:react_on_slow_down_bonus()
	 elseif other.game_object.bonustype == 'bonus_red' then
	    balls:react_on_accelerate_bonus()
	 elseif other.game_object.bonustype == 'bonus_orange' then
	    platform:react_on_increase_bonus()
	    balls:unstick_all_balls( platform )
	 elseif other.game_object.bonustype == 'bonus_yellow' then
	    platform:react_on_decrease_bonus()
	    balls:unstick_all_balls( platform )
	 elseif other.game_object.bonustype == 'bonus_green' then
	    platform:react_on_glue_bonus()
	 elseif other.game_object.bonustype == 'bonus_purple' then
	    balls:react_on_new_ball_bonus()
	 elseif other.game_object.bonustype == 'bonus_grey' then
	    walls:react_on_next_level_bonus()
	 elseif other.game_object.bonustype == 'bonus_white' then
	    life_display:add_new_life()
	 end
	 other.game_object:react_on_platform_collision(
	    platform.collisions.shape,
	    (-1) * vector( separating_vector.x, separating_vector.y ) )
      end
   end

   -- Ball
   for _, ball in pairs( balls.balls ) do
      local collisions = collider:collisions( ball.collisions.shape )
      for other, separating_vector in pairs( collisions ) do
	 if other.game_object.name == "wall" then
	    ball:react_on_wall_collision( other, separating_vector )
	 elseif other.game_object.name == "platform" then
	    ball:react_on_platform_collision( other, separating_vector )
	 elseif other.game_object.name == "brick" then
	    ball:react_on_brick_collision( other, separating_vector )
	    other.game_object:react_on_ball_collision(
	       ball.collisions.shape,
	       (-1) * vector( separating_vector.x, separating_vector.y )  )
	    score_display:react_on_ball_brick_collision( other )
	 end
      end
   end
end	


function game:react_on_next_level_bonus()
   local next_level = level_number + 1 
   if next_level <= #level_sequence.sequence then
      Gamestate.switch( game, { level_sequence = level_sequence,
				level_number = next_level,
				life_display = life_display,
				score_display = score_display } )
   else
      Gamestate.switch( gamefinished )
   end	
end


return game
